# © 2019-present nextmv.io inc

"""
Schema (class) definitions for the entities in Nextroute.
"""

from .input import Defaults as Defaults
from .input import DurationGroup as DurationGroup
from .input import Input as Input
from .location import Location as Location
from .output import ObjectiveOutput as ObjectiveOutput
from .output import Output as Output
from .output import PlannedStopOutput as PlannedStopOutput
from .output import Solution as Solution
from .output import StopOutput as StopOutput
from .output import VehicleOutput as VehicleOutput
from .output import Version as Version
from .statistics import DataPoint as DataPoint
from .statistics import ResultStatistics as ResultStatistics
from .statistics import RunStatistics as RunStatistics
from .statistics import Series as Series
from .statistics import SeriesData as SeriesData
from .statistics import Statistics as Statistics
from .stop import AlternateStop as AlternateStop
from .stop import Stop as Stop
from .stop import StopDefaults as StopDefaults
from .vehicle import InitialStop as InitialStop
from .vehicle import Vehicle as Vehicle
from .vehicle import VehicleDefaults as VehicleDefaults
